#include <commscontroller.h>
#include <iostream>
#include <time.h>
#include <qapplication.h>

/* Constructor */
CommsController::CommsController(Genboard *myGenboard) {
    this->myGenboard = myGenboard;

    mySerialPort = new QextSerialPort();
    mySerialPort->setBaudRate(BAUD9600); 
    mySerialPort->setDataBits(DATA_8); 
    mySerialPort->setStopBits(STOP_1); 
    mySerialPort->setFlowControl(FLOW_OFF); 
    mySerialPort->setParity(PAR_NONE);
    mySerialPort->open();
 
}

/* Destructor */
CommsController::~CommsController() {
    mySerialPort->flush();
    mySerialPort->close();
    delete mySerialPort;
}

/* Thread start point */
void CommsController::run() {
	while (mySerialPort->isOpen()) {
//	while (true) {
	    this->loadRealtimeVars();
	    usleep(250);
	}
	std::cout << "mySerialPort is not open.\n";
}


/* Load the realtime variables into the buffer */
void CommsController::loadRealtimeVars() {

	/*
    // fake some data
    int value;
    srand(time(NULL));
    for (int counter = 0; counter < 22; counter++) {
    	value = (int)(255.0*rand()/(RAND_MAX + 1.0));
	myGenboard->setRealtimeVar(counter, value);
    }
	*/
    
    // Ask the ECU for realtime variables
    mySerialPort->putch((unsigned char)'A');
    //mySerialPort->flush();
    //waitForData();
    
    // Copy the values into the buffer 
    int value;
    for (int counter = 0; counter < 22; counter++) {
	this->waitForData();
    	value = mySerialPort->getch();
	myGenboard->setRealtimeVar(counter, value);
    };

}

/* Sleep until the ECU responds, or a second has elapsed */
void CommsController::waitForData() {
//	int timeout = 0;
	while (mySerialPort->bytesWaiting() < 1) {
		usleep(100);
//		timeout++;
	}	
}

